

{- |
Copyright  : Will Thompson, Iñaki García Etxebarria and Jonas Platte
License    : LGPL-2.1
Maintainer : Iñaki García Etxebarria (inaki@blueleaf.cc)

The 'GI.GstVideo.Interfaces.VideoOverlay.VideoOverlay' interface is used for 2 main purposes :

* To get a grab on the Window where the video sink element is going to render.
  This is achieved by either being informed about the Window identifier that
  the video sink element generated, or by forcing the video sink element to use
  a specific Window identifier for rendering.
* To force a redrawing of the latest video frame the video sink element
  displayed on the Window. Indeed if the 'GI.Gst.Objects.Pipeline.Pipeline' is in @/GST_STATE_PAUSED/@
  state, moving the Window around will damage its content. Application
  developers will want to handle the Expose events themselves and force the
  video sink element to refresh the Window\'s content.

Using the Window created by the video sink is probably the simplest scenario,
in some cases, though, it might not be flexible enough for application
developers if they need to catch events such as mouse moves and button
clicks.

Setting a specific Window identifier on the video sink element is the most
flexible solution but it has some issues. Indeed the application needs to set
its Window identifier at the right time to avoid internal Window creation
from the video sink element. To solve this issue a 'GI.Gst.Structs.Message.Message' is posted on
the bus to inform the application that it should set the Window identifier
immediately. Here is an example on how to do that correctly:
>
>static GstBusSyncReply
>create_window (GstBus * bus, GstMessage * message, GstPipeline * pipeline)
>{
> // ignore anything but 'prepare-window-handle' element messages
> if (!gst_is_video_overlay_prepare_window_handle_message (message))
>   return GST_BUS_PASS;
>
> win = XCreateSimpleWindow (disp, root, 0, 0, 320, 240, 0, 0, 0);
>
> XSetWindowBackgroundPixmap (disp, win, None);
>
> XMapRaised (disp, win);
>
> XSync (disp, FALSE);
>
> gst_video_overlay_set_window_handle (GST_VIDEO_OVERLAY (GST_MESSAGE_SRC (message)),
>     win);
>
> gst_message_unref (message);
>
> return GST_BUS_DROP;
>}
>...
>int
>main (int argc, char **argv)
>{
>...
> bus = gst_pipeline_get_bus (GST_PIPELINE (pipeline));
> gst_bus_set_sync_handler (bus, (GstBusSyncHandler) create_window, pipeline,
>        NULL);
>...
>}


== Two basic usage scenarios

There are two basic usage scenarios: in the simplest case, the application
uses @/playbin/@ or @/plasink/@ or knows exactly what particular element is used
for video output, which is usually the case when the application creates
the videosink to use (e.g. @/xvimagesink/@, @/ximagesink/@, etc.) itself; in this
case, the application can just create the videosink element, create and
realize the window to render the video on and then
call 'GI.GstVideo.Interfaces.VideoOverlay.videoOverlaySetWindowHandle' directly with the XID or native
window handle, before starting up the pipeline.
As @/playbin/@ and @/playsink/@ implement the video overlay interface and proxy
it transparently to the actual video sink even if it is created later, this
case also applies when using these elements.

In the other and more common case, the application does not know in advance
what GStreamer video sink element will be used for video output. This is
usually the case when an element such as @/autovideosink/@ is used.
In this case, the video sink element itself is created
asynchronously from a GStreamer streaming thread some time after the
pipeline has been started up. When that happens, however, the video sink
will need to know right then whether to render onto an already existing
application window or whether to create its own window. This is when it
posts a prepare-window-handle message, and that is also why this message needs
to be handled in a sync bus handler which will be called from the streaming
thread directly (because the video sink will need an answer right then).

As response to the prepare-window-handle element message in the bus sync
handler, the application may use 'GI.GstVideo.Interfaces.VideoOverlay.videoOverlaySetWindowHandle' to tell
the video sink to render onto an existing window surface. At this point the
application should already have obtained the window handle \/ XID, so it
just needs to set it. It is generally not advisable to call any GUI toolkit
functions or window system functions from the streaming thread in which the
prepare-window-handle message is handled, because most GUI toolkits and
windowing systems are not thread-safe at all and a lot of care would be
required to co-ordinate the toolkit and window system calls of the
different threads (Gtk+ users please note: prior to Gtk+ 2.18
@/GDK_WINDOW_XID()/@ was just a simple structure access, so generally fine to do
within the bus sync handler; this macro was changed to a function call in
Gtk+ 2.18 and later, which is likely to cause problems when called from a
sync handler; see below for a better approach without @/GDK_WINDOW_XID()/@
used in the callback).

== GstVideoOverlay and Gtk+

>
>#include &lt;gst/video/videooverlay.h&gt;
>#include &lt;gtk/gtk.h&gt;
>#ifdef GDK_WINDOWING_X11
>#include &lt;gdk/gdkx.h&gt;  // for GDK_WINDOW_XID
>#endif
>#ifdef GDK_WINDOWING_WIN32
>#include &lt;gdk/gdkwin32.h&gt;  // for GDK_WINDOW_HWND
>#endif
>...
>static guintptr video_window_handle = 0;
>...
>static GstBusSyncReply
>bus_sync_handler (GstBus * bus, GstMessage * message, gpointer user_data)
>{
> // ignore anything but 'prepare-window-handle' element messages
> if (!gst_is_video_overlay_prepare_window_handle_message (message))
>   return GST_BUS_PASS;
>
> if (video_window_handle != 0) {
>   GstVideoOverlay *overlay;
>
>   // GST_MESSAGE_SRC (message) will be the video sink element
>   overlay = GST_VIDEO_OVERLAY (GST_MESSAGE_SRC (message));
>   gst_video_overlay_set_window_handle (overlay, video_window_handle);
> } else {
>   g_warning ("Should have obtained video_window_handle by now!");
> }
>
> gst_message_unref (message);
> return GST_BUS_DROP;
>}
>...
>static void
>video_widget_realize_cb (GtkWidget * widget, gpointer data)
>{
>#if GTK_CHECK_VERSION(2,18,0)
>  // Tell Gtk+/Gdk to create a native window for this widget instead of
>  // drawing onto the parent widget.
>  // This is here just for pedagogical purposes, GDK_WINDOW_XID will call
>  // it as well in newer Gtk versions
>  if (!gdk_window_ensure_native (widget->window))
>    g_error ("Couldn't create native window needed for GstVideoOverlay!");
>#endif
>
>#ifdef GDK_WINDOWING_X11
>  {
>    gulong xid = GDK_WINDOW_XID (gtk_widget_get_window (video_window));
>    video_window_handle = xid;
>  }
>#endif
>#ifdef GDK_WINDOWING_WIN32
>  {
>    HWND wnd = GDK_WINDOW_HWND (gtk_widget_get_window (video_window));
>    video_window_handle = (guintptr) wnd;
>  }
>#endif
>}
>...
>int
>main (int argc, char **argv)
>{
>  GtkWidget *video_window;
>  GtkWidget *app_window;
>  ...
>  app_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
>  ...
>  video_window = gtk_drawing_area_new ();
>  g_signal_connect (video_window, "realize",
>      G_CALLBACK (video_widget_realize_cb), NULL);
>  gtk_widget_set_double_buffered (video_window, FALSE);
>  ...
>  // usually the video_window will not be directly embedded into the
>  // application window like this, but there will be many other widgets
>  // and the video window will be embedded in one of them instead
>  gtk_container_add (GTK_CONTAINER (ap_window), video_window);
>  ...
>  // show the GUI
>  gtk_widget_show_all (app_window);
>
>  // realize window now so that the video window gets created and we can
>  // obtain its XID/HWND before the pipeline is started up and the videosink
>  // asks for the XID/HWND of the window to render onto
>  gtk_widget_realize (video_window);
>
>  // we should have the XID/HWND now
>  g_assert (video_window_handle != 0);
>  ...
>  // set up sync handler for setting the xid once the pipeline is started
>  bus = gst_pipeline_get_bus (GST_PIPELINE (pipeline));
>  gst_bus_set_sync_handler (bus, (GstBusSyncHandler) bus_sync_handler, NULL,
>      NULL);
>  gst_object_unref (bus);
>  ...
>  gst_element_set_state (pipeline, GST_STATE_PLAYING);
>  ...
>}


== GstVideoOverlay and Qt

>
>#include &lt;glib.h&gt;
>#include &lt;gst/gst.h&gt;
>#include &lt;gst/video/videooverlay.h&gt;
>
>#include &lt;QApplication&gt;
>#include &lt;QTimer&gt;
>#include &lt;QWidget&gt;
>
>int main(int argc, char *argv[])
>{
>  if (!g_thread_supported ())
>    g_thread_init (NULL);
>
>  gst_init (&argc, &argv);
>  QApplication app(argc, argv);
>  app.connect(&app, SIGNAL(lastWindowClosed()), &app, SLOT(quit ()));
>
>  // prepare the pipeline
>
>  GstElement *pipeline = gst_pipeline_new ("xvoverlay");
>  GstElement *src = gst_element_factory_make ("videotestsrc", NULL);
>  GstElement *sink = gst_element_factory_make ("xvimagesink", NULL);
>  gst_bin_add_many (GST_BIN (pipeline), src, sink, NULL);
>  gst_element_link (src, sink);
>
>  // prepare the ui
>
>  QWidget window;
>  window.resize(320, 240);
>  window.show();
>
>  WId xwinid = window.winId();
>  gst_video_overlay_set_window_handle (GST_VIDEO_OVERLAY (sink), xwinid);
>
>  // run the pipeline
>
>  GstStateChangeReturn sret = gst_element_set_state (pipeline,
>      GST_STATE_PLAYING);
>  if (sret == GST_STATE_CHANGE_FAILURE) {
>    gst_element_set_state (pipeline, GST_STATE_NULL);
>    gst_object_unref (pipeline);
>    // Exit application
>    QTimer::singleShot(0, QApplication::activeWindow(), SLOT(quit()));
>  }
>
>  int ret = app.exec();
>
>  window.hide();
>  gst_element_set_state (pipeline, GST_STATE_NULL);
>  gst_object_unref (pipeline);
>
>  return ret;
>}

-}

#define ENABLE_OVERLOADING (MIN_VERSION_haskell_gi_overloading(1,0,0) \
       && !defined(__HADDOCK_VERSION__))

module GI.GstVideo.Interfaces.VideoOverlay
    ( 

-- * Exported types
    VideoOverlay(..)                        ,
    noVideoOverlay                          ,
    IsVideoOverlay                          ,


 -- * Methods
-- ** expose #method:expose#

#if ENABLE_OVERLOADING
    VideoOverlayExposeMethodInfo            ,
#endif
    videoOverlayExpose                      ,


-- ** gotWindowHandle #method:gotWindowHandle#

#if ENABLE_OVERLOADING
    VideoOverlayGotWindowHandleMethodInfo   ,
#endif
    videoOverlayGotWindowHandle             ,


-- ** handleEvents #method:handleEvents#

#if ENABLE_OVERLOADING
    VideoOverlayHandleEventsMethodInfo      ,
#endif
    videoOverlayHandleEvents                ,


-- ** prepareWindowHandle #method:prepareWindowHandle#

#if ENABLE_OVERLOADING
    VideoOverlayPrepareWindowHandleMethodInfo,
#endif
    videoOverlayPrepareWindowHandle         ,


-- ** setProperty #method:setProperty#

    videoOverlaySetProperty                 ,


-- ** setRenderRectangle #method:setRenderRectangle#

#if ENABLE_OVERLOADING
    VideoOverlaySetRenderRectangleMethodInfo,
#endif
    videoOverlaySetRenderRectangle          ,


-- ** setWindowHandle #method:setWindowHandle#

#if ENABLE_OVERLOADING
    VideoOverlaySetWindowHandleMethodInfo   ,
#endif
    videoOverlaySetWindowHandle             ,




    ) where

import Data.GI.Base.ShortPrelude
import qualified Data.GI.Base.ShortPrelude as SP
import qualified Data.GI.Base.Overloading as O
import qualified Prelude as P

import qualified Data.GI.Base.Attributes as GI.Attributes
import qualified Data.GI.Base.ManagedPtr as B.ManagedPtr
import qualified Data.GI.Base.GClosure as B.GClosure
import qualified Data.GI.Base.GError as B.GError
import qualified Data.GI.Base.GVariant as B.GVariant
import qualified Data.GI.Base.GValue as B.GValue
import qualified Data.GI.Base.GParamSpec as B.GParamSpec
import qualified Data.GI.Base.CallStack as B.CallStack
import qualified Data.GI.Base.Properties as B.Properties
import qualified Data.Text as T
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map
import qualified Foreign.Ptr as FP
import qualified GHC.OverloadedLabels as OL

import qualified GI.GObject.Objects.Object as GObject.Object

-- interface VideoOverlay 
-- | Memory-managed wrapper type.
newtype VideoOverlay = VideoOverlay (ManagedPtr VideoOverlay)
-- | A convenience alias for `Nothing` :: `Maybe` `VideoOverlay`.
noVideoOverlay :: Maybe VideoOverlay
noVideoOverlay = Nothing

#if ENABLE_OVERLOADING
type instance O.SignalList VideoOverlay = VideoOverlaySignalList
type VideoOverlaySignalList = ('[ ] :: [(Symbol, *)])

#endif

-- | Type class for types which implement `VideoOverlay`.
class ManagedPtrNewtype a => IsVideoOverlay a
instance IsVideoOverlay VideoOverlay
-- XXX Wrapping a foreign struct/union with no known destructor or size, leak?
instance WrappedPtr VideoOverlay where
    wrappedPtrCalloc = return nullPtr
    wrappedPtrCopy = return
    wrappedPtrFree = Nothing


#if ENABLE_OVERLOADING
type family ResolveVideoOverlayMethod (t :: Symbol) (o :: *) :: * where
    ResolveVideoOverlayMethod "expose" o = VideoOverlayExposeMethodInfo
    ResolveVideoOverlayMethod "gotWindowHandle" o = VideoOverlayGotWindowHandleMethodInfo
    ResolveVideoOverlayMethod "handleEvents" o = VideoOverlayHandleEventsMethodInfo
    ResolveVideoOverlayMethod "prepareWindowHandle" o = VideoOverlayPrepareWindowHandleMethodInfo
    ResolveVideoOverlayMethod "setRenderRectangle" o = VideoOverlaySetRenderRectangleMethodInfo
    ResolveVideoOverlayMethod "setWindowHandle" o = VideoOverlaySetWindowHandleMethodInfo
    ResolveVideoOverlayMethod l o = O.MethodResolutionFailed l o

instance (info ~ ResolveVideoOverlayMethod t VideoOverlay, O.MethodInfo info VideoOverlay p) => OL.IsLabel t (VideoOverlay -> p) where
#if MIN_VERSION_base(4,10,0)
    fromLabel = O.overloadedMethod (O.MethodProxy :: O.MethodProxy info)
#else
    fromLabel _ = O.overloadedMethod (O.MethodProxy :: O.MethodProxy info)
#endif

#endif

-- method VideoOverlay::expose
-- method type : OrdinaryMethod
-- Args : [Arg {argCName = "overlay", argType = TInterface (Name {namespace = "GstVideo", name = "VideoOverlay"}), direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "a #GstVideoOverlay to expose.", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing}]
-- Lengths : []
-- returnType : Nothing
-- throws : False
-- Skip return : False

foreign import ccall "gst_video_overlay_expose" gst_video_overlay_expose :: 
    Ptr VideoOverlay ->                     -- overlay : TInterface (Name {namespace = "GstVideo", name = "VideoOverlay"})
    IO ()

{- |
Tell an overlay that it has been exposed. This will redraw the current frame
in the drawable even if the pipeline is PAUSED.
-}
videoOverlayExpose ::
    (B.CallStack.HasCallStack, MonadIO m, IsVideoOverlay a) =>
    a
    {- ^ /@overlay@/: a 'GI.GstVideo.Interfaces.VideoOverlay.VideoOverlay' to expose. -}
    -> m ()
videoOverlayExpose overlay = liftIO $ do
    overlay' <- unsafeManagedPtrCastPtr overlay
    gst_video_overlay_expose overlay'
    touchManagedPtr overlay
    return ()

#if ENABLE_OVERLOADING
data VideoOverlayExposeMethodInfo
instance (signature ~ (m ()), MonadIO m, IsVideoOverlay a) => O.MethodInfo VideoOverlayExposeMethodInfo a signature where
    overloadedMethod _ = videoOverlayExpose

#endif

-- method VideoOverlay::got_window_handle
-- method type : OrdinaryMethod
-- Args : [Arg {argCName = "overlay", argType = TInterface (Name {namespace = "GstVideo", name = "VideoOverlay"}), direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "a #GstVideoOverlay which got a window", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing},Arg {argCName = "handle", argType = TBasicType TUIntPtr, direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "a platform-specific handle referencing the window", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing}]
-- Lengths : []
-- returnType : Nothing
-- throws : False
-- Skip return : False

foreign import ccall "gst_video_overlay_got_window_handle" gst_video_overlay_got_window_handle :: 
    Ptr VideoOverlay ->                     -- overlay : TInterface (Name {namespace = "GstVideo", name = "VideoOverlay"})
    CUIntPtr ->                             -- handle : TBasicType TUIntPtr
    IO ()

{- |
This will post a \"have-window-handle\" element message on the bus.

This function should only be used by video overlay plugin developers.
-}
videoOverlayGotWindowHandle ::
    (B.CallStack.HasCallStack, MonadIO m, IsVideoOverlay a) =>
    a
    {- ^ /@overlay@/: a 'GI.GstVideo.Interfaces.VideoOverlay.VideoOverlay' which got a window -}
    -> CUIntPtr
    {- ^ /@handle@/: a platform-specific handle referencing the window -}
    -> m ()
videoOverlayGotWindowHandle overlay handle = liftIO $ do
    overlay' <- unsafeManagedPtrCastPtr overlay
    gst_video_overlay_got_window_handle overlay' handle
    touchManagedPtr overlay
    return ()

#if ENABLE_OVERLOADING
data VideoOverlayGotWindowHandleMethodInfo
instance (signature ~ (CUIntPtr -> m ()), MonadIO m, IsVideoOverlay a) => O.MethodInfo VideoOverlayGotWindowHandleMethodInfo a signature where
    overloadedMethod _ = videoOverlayGotWindowHandle

#endif

-- method VideoOverlay::handle_events
-- method type : OrdinaryMethod
-- Args : [Arg {argCName = "overlay", argType = TInterface (Name {namespace = "GstVideo", name = "VideoOverlay"}), direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "a #GstVideoOverlay to expose.", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing},Arg {argCName = "handle_events", argType = TBasicType TBoolean, direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "a #gboolean indicating if events should be handled or not.", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing}]
-- Lengths : []
-- returnType : Nothing
-- throws : False
-- Skip return : False

foreign import ccall "gst_video_overlay_handle_events" gst_video_overlay_handle_events :: 
    Ptr VideoOverlay ->                     -- overlay : TInterface (Name {namespace = "GstVideo", name = "VideoOverlay"})
    CInt ->                                 -- handle_events : TBasicType TBoolean
    IO ()

{- |
Tell an overlay that it should handle events from the window system. These
events are forwarded upstream as navigation events. In some window system,
events are not propagated in the window hierarchy if a client is listening
for them. This method allows you to disable events handling completely
from the 'GI.GstVideo.Interfaces.VideoOverlay.VideoOverlay'.
-}
videoOverlayHandleEvents ::
    (B.CallStack.HasCallStack, MonadIO m, IsVideoOverlay a) =>
    a
    {- ^ /@overlay@/: a 'GI.GstVideo.Interfaces.VideoOverlay.VideoOverlay' to expose. -}
    -> Bool
    {- ^ /@handleEvents@/: a 'Bool' indicating if events should be handled or not. -}
    -> m ()
videoOverlayHandleEvents overlay handleEvents = liftIO $ do
    overlay' <- unsafeManagedPtrCastPtr overlay
    let handleEvents' = (fromIntegral . fromEnum) handleEvents
    gst_video_overlay_handle_events overlay' handleEvents'
    touchManagedPtr overlay
    return ()

#if ENABLE_OVERLOADING
data VideoOverlayHandleEventsMethodInfo
instance (signature ~ (Bool -> m ()), MonadIO m, IsVideoOverlay a) => O.MethodInfo VideoOverlayHandleEventsMethodInfo a signature where
    overloadedMethod _ = videoOverlayHandleEvents

#endif

-- method VideoOverlay::prepare_window_handle
-- method type : OrdinaryMethod
-- Args : [Arg {argCName = "overlay", argType = TInterface (Name {namespace = "GstVideo", name = "VideoOverlay"}), direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "a #GstVideoOverlay which does not yet have an Window handle set", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing}]
-- Lengths : []
-- returnType : Nothing
-- throws : False
-- Skip return : False

foreign import ccall "gst_video_overlay_prepare_window_handle" gst_video_overlay_prepare_window_handle :: 
    Ptr VideoOverlay ->                     -- overlay : TInterface (Name {namespace = "GstVideo", name = "VideoOverlay"})
    IO ()

{- |
This will post a \"prepare-window-handle\" element message on the bus
to give applications an opportunity to call
'GI.GstVideo.Interfaces.VideoOverlay.videoOverlaySetWindowHandle' before a plugin creates its own
window.

This function should only be used by video overlay plugin developers.
-}
videoOverlayPrepareWindowHandle ::
    (B.CallStack.HasCallStack, MonadIO m, IsVideoOverlay a) =>
    a
    {- ^ /@overlay@/: a 'GI.GstVideo.Interfaces.VideoOverlay.VideoOverlay' which does not yet have an Window handle set -}
    -> m ()
videoOverlayPrepareWindowHandle overlay = liftIO $ do
    overlay' <- unsafeManagedPtrCastPtr overlay
    gst_video_overlay_prepare_window_handle overlay'
    touchManagedPtr overlay
    return ()

#if ENABLE_OVERLOADING
data VideoOverlayPrepareWindowHandleMethodInfo
instance (signature ~ (m ()), MonadIO m, IsVideoOverlay a) => O.MethodInfo VideoOverlayPrepareWindowHandleMethodInfo a signature where
    overloadedMethod _ = videoOverlayPrepareWindowHandle

#endif

-- method VideoOverlay::set_render_rectangle
-- method type : OrdinaryMethod
-- Args : [Arg {argCName = "overlay", argType = TInterface (Name {namespace = "GstVideo", name = "VideoOverlay"}), direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "a #GstVideoOverlay", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing},Arg {argCName = "x", argType = TBasicType TInt, direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "the horizontal offset of the render area inside the window", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing},Arg {argCName = "y", argType = TBasicType TInt, direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "the vertical offset of the render area inside the window", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing},Arg {argCName = "width", argType = TBasicType TInt, direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "the width of the render area inside the window", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing},Arg {argCName = "height", argType = TBasicType TInt, direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "the height of the render area inside the window", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing}]
-- Lengths : []
-- returnType : Just (TBasicType TBoolean)
-- throws : False
-- Skip return : False

foreign import ccall "gst_video_overlay_set_render_rectangle" gst_video_overlay_set_render_rectangle :: 
    Ptr VideoOverlay ->                     -- overlay : TInterface (Name {namespace = "GstVideo", name = "VideoOverlay"})
    Int32 ->                                -- x : TBasicType TInt
    Int32 ->                                -- y : TBasicType TInt
    Int32 ->                                -- width : TBasicType TInt
    Int32 ->                                -- height : TBasicType TInt
    IO CInt

{- |
Configure a subregion as a video target within the window set by
'GI.GstVideo.Interfaces.VideoOverlay.videoOverlaySetWindowHandle'. If this is not used or not supported
the video will fill the area of the window set as the overlay to 100%.
By specifying the rectangle, the video can be overlayed to a specific region
of that window only. After setting the new rectangle one should call
'GI.GstVideo.Interfaces.VideoOverlay.videoOverlayExpose' to force a redraw. To unset the region pass -1 for
the /@width@/ and /@height@/ parameters.

This method is needed for non fullscreen video overlay in UI toolkits that
do not support subwindows.
-}
videoOverlaySetRenderRectangle ::
    (B.CallStack.HasCallStack, MonadIO m, IsVideoOverlay a) =>
    a
    {- ^ /@overlay@/: a 'GI.GstVideo.Interfaces.VideoOverlay.VideoOverlay' -}
    -> Int32
    {- ^ /@x@/: the horizontal offset of the render area inside the window -}
    -> Int32
    {- ^ /@y@/: the vertical offset of the render area inside the window -}
    -> Int32
    {- ^ /@width@/: the width of the render area inside the window -}
    -> Int32
    {- ^ /@height@/: the height of the render area inside the window -}
    -> m Bool
    {- ^ __Returns:__ 'False' if not supported by the sink. -}
videoOverlaySetRenderRectangle overlay x y width height = liftIO $ do
    overlay' <- unsafeManagedPtrCastPtr overlay
    result <- gst_video_overlay_set_render_rectangle overlay' x y width height
    let result' = (/= 0) result
    touchManagedPtr overlay
    return result'

#if ENABLE_OVERLOADING
data VideoOverlaySetRenderRectangleMethodInfo
instance (signature ~ (Int32 -> Int32 -> Int32 -> Int32 -> m Bool), MonadIO m, IsVideoOverlay a) => O.MethodInfo VideoOverlaySetRenderRectangleMethodInfo a signature where
    overloadedMethod _ = videoOverlaySetRenderRectangle

#endif

-- method VideoOverlay::set_window_handle
-- method type : OrdinaryMethod
-- Args : [Arg {argCName = "overlay", argType = TInterface (Name {namespace = "GstVideo", name = "VideoOverlay"}), direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "a #GstVideoOverlay to set the window on.", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing},Arg {argCName = "handle", argType = TBasicType TUIntPtr, direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "a handle referencing the window.", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing}]
-- Lengths : []
-- returnType : Nothing
-- throws : False
-- Skip return : False

foreign import ccall "gst_video_overlay_set_window_handle" gst_video_overlay_set_window_handle :: 
    Ptr VideoOverlay ->                     -- overlay : TInterface (Name {namespace = "GstVideo", name = "VideoOverlay"})
    CUIntPtr ->                             -- handle : TBasicType TUIntPtr
    IO ()

{- |
This will call the video overlay\'s set_window_handle method. You
should use this method to tell to an overlay to display video output to a
specific window (e.g. an XWindow on X11). Passing 0 as the  /@handle@/ will
tell the overlay to stop using that window and create an internal one.
-}
videoOverlaySetWindowHandle ::
    (B.CallStack.HasCallStack, MonadIO m, IsVideoOverlay a) =>
    a
    {- ^ /@overlay@/: a 'GI.GstVideo.Interfaces.VideoOverlay.VideoOverlay' to set the window on. -}
    -> CUIntPtr
    {- ^ /@handle@/: a handle referencing the window. -}
    -> m ()
videoOverlaySetWindowHandle overlay handle = liftIO $ do
    overlay' <- unsafeManagedPtrCastPtr overlay
    gst_video_overlay_set_window_handle overlay' handle
    touchManagedPtr overlay
    return ()

#if ENABLE_OVERLOADING
data VideoOverlaySetWindowHandleMethodInfo
instance (signature ~ (CUIntPtr -> m ()), MonadIO m, IsVideoOverlay a) => O.MethodInfo VideoOverlaySetWindowHandleMethodInfo a signature where
    overloadedMethod _ = videoOverlaySetWindowHandle

#endif

-- method VideoOverlay::set_property
-- method type : MemberFunction
-- Args : [Arg {argCName = "object", argType = TInterface (Name {namespace = "GObject", name = "Object"}), direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "The instance on which the property is set", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing},Arg {argCName = "last_prop_id", argType = TBasicType TInt, direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "The highest property ID.", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing},Arg {argCName = "property_id", argType = TBasicType TUInt, direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "The property ID", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing},Arg {argCName = "value", argType = TInterface (Name {namespace = "GObject", name = "Value"}), direction = DirectionIn, mayBeNull = False, argDoc = Documentation {rawDocText = Just "The #GValue to be set", sinceVersion = Nothing}, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing}]
-- Lengths : []
-- returnType : Just (TBasicType TBoolean)
-- throws : False
-- Skip return : False

foreign import ccall "gst_video_overlay_set_property" gst_video_overlay_set_property :: 
    Ptr GObject.Object.Object ->            -- object : TInterface (Name {namespace = "GObject", name = "Object"})
    Int32 ->                                -- last_prop_id : TBasicType TInt
    Word32 ->                               -- property_id : TBasicType TUInt
    Ptr GValue ->                           -- value : TInterface (Name {namespace = "GObject", name = "Value"})
    IO CInt

{- |
This helper shall be used by classes implementing the 'GI.GstVideo.Interfaces.VideoOverlay.VideoOverlay'
interface that want the render rectangle to be controllable using
properties. This helper will parse and set the render rectangle calling
'GI.GstVideo.Interfaces.VideoOverlay.videoOverlaySetRenderRectangle'.
-}
videoOverlaySetProperty ::
    (B.CallStack.HasCallStack, MonadIO m, GObject.Object.IsObject a) =>
    a
    {- ^ /@object@/: The instance on which the property is set -}
    -> Int32
    {- ^ /@lastPropId@/: The highest property ID. -}
    -> Word32
    {- ^ /@propertyId@/: The property ID -}
    -> GValue
    {- ^ /@value@/: The 'GI.GObject.Structs.Value.Value' to be set -}
    -> m Bool
    {- ^ __Returns:__ 'True' if the /@propertyId@/ matches the GstVideoOverlay property

Since 1.14 -}
videoOverlaySetProperty object lastPropId propertyId value = liftIO $ do
    object' <- unsafeManagedPtrCastPtr object
    value' <- unsafeManagedPtrGetPtr value
    result <- gst_video_overlay_set_property object' lastPropId propertyId value'
    let result' = (/= 0) result
    touchManagedPtr object
    touchManagedPtr value
    return result'

#if ENABLE_OVERLOADING
#endif


